echo on
pause(1)
clc
% This demo describes certain preconditioning methods.
% All experiments here are in ONE DIMENSION. This code
% does not clear the screen, so it's recommended that
% you run it from matlabtool called from a commandtool.
%
%
%
%   First we load the data required for preconditioning. This is 
%   a file containing essentially the coefficients of the cardinal
%   function for the multiquadric phi(r) = sqrt(1 + r^2) based on 
%   the integers.
%   
load w.dat
%
%   Our first problem will be quite small - only 128 data points.
%   Note that we only store the first row of the distance matrix.
%
n = 128;
a = phi([0:n-1]');
pause  % strike a key to continue
%
%   We now construct the preconditioner
%
m = 9;
c = fmakec(w, m, n);
%
%   Our righthandside will be a mess, so that you know there's
%   nothing fishy going on. 
%
b = rand(n, 1);
pause  % strike a key to continue
%
%   Now we solve the linear system using the preconditioner
%   constructed above.
%
xfpcg = fpcg(a, b, zeros(n,1), 1.0e-13, c);
%
%   Now we compare this to the  preconditioning method of Dyn et al.
%    
m = 1;
c = fmakec(w, m, n);
pause  % strike a key to continue

xfpcg = fpcg(a, b, zeros(n,1), 1.0e-13, c);
%
%   The behaviour shown here is typical. 
%
pause  % strike a key to continue

%
%   We now deal with a decent problem. Let the number
%   data points be 4096. We use my preconditioner and
%   a random righthandside.
%
n = 4096;
a = phi([0:n-1]');   
m = 9;
c = fmakec(w, m, n);
b = rand(n, 1);
%
%   The following run takes about 1.5 minutes on a Sparc 1.
%
pause  % strike a key to continue

xfpcg = fpcg(a, b, zeros(n,1), 1.0e-13, c);
pause  % strike a key to continue

%
%   The rough estimate of 1.5 minutes for 4000 points is
%   a good guide to the performance of this code on a 
%   Sparc 1. Of course, the workload is O(n log n), not
%   O(n), but a more precise estimate would be more 
%   obscure. At the end of the demonstration, we solve
%   a system with 16384 points, which takes just over
%   6 minutes (assuming it's done in RAM rather than
%   being swapped to disc).
%
%   We now investigate how well my preconditioner
%   and Nira's preconditioner concentrate the
%   eigenvalues near one. This part of the demo
%   does NOT use FFT, and therefore we return to
%   the toy problem size of 128 points. 
% 
clear
load w.dat
pause  % strike a key to continue


n = 128;
a = toeplitz(phi(0:n-1));
%
%   We first calculate the matrix C for Nira's preconditioner
%   and then display the eigenvalues of C * a.
%
m  = 1;
C = makec(w, m, n);
Ca = C * a;
ev = sort(eig(Ca));
plot(ev(2:n-1), 'o');
%
%   This curve has a rather nice equation; let me know if you 
%   want to know more.
%
pause  % strike a key to continue
%
%   Now we use my preconditioner...
%
m = 9;
C = makec(w, m, n);
Ca = C * a;
ev = sort(eig(Ca));
plot(ev(2:n-1), 'o');
pause  % strike a key to continue
%
%   ...which is clumping the spectrum rather well. 
%
%   We now return to solving linear systems:
%   The final example chooses n = 2^14 = 16384. This example
%   takes about 6-7 minutes on a Sparc 1.
%
n = 2^14;
m = 9;
a = phi([0:n-1]');
c = fmakec(w, m, n);
b = rand(n, 1);
xfpcg = fpcg(a, b, zeros(n,1), 1.0e-13, c);
%
%   and that's it.
%
